import 'package:json_annotation/json_annotation.dart';

part 'initializer.g.dart';

/// An annotation used to specify a variable/function/field/method to generate code for.
@JsonSerializable(
  checked: true,
  disallowUnrecognizedKeys: true,
  fieldRename: FieldRename.snake,
)
class Initializer {
  /// Enable debug mode
  ///
  /// **NOTE: only works when setting in build.yaml**
  /// ```yaml
  /// targets:
  ///   $default:
  ///     builders:
  ///       initializer:
  ///         options:
  ///           debug: true
  /// ```
  final bool debug;

  /// Split initializer code into different groups by name.
  /// default [group] name is "default"
  final String? group;

  /// Eet the path of generated output file.
  /// Default [outputPath] is `lib/src/init.init.dart`
  /// **NOTE: only works when setting in build.yaml**
  /// ```yaml
  /// targets:
  ///   $default:
  ///     builders:
  ///       initializer:
  ///         options:
  ///           output_path: lib/init.init.dart
  /// ```
  final String? outputPath;

  /// sort initializer order by group name
  /// **NOTE: only works when setting in build.yaml**
  /// ```yaml
  /// targets:
  ///   $default:
  ///     builders:
  ///       initializer:
  ///         options:
  ///           order:
  ///             - default
  ///             - group1
  ///             - group2
  /// ```
  final Set<String>? order;

  /// Creates a new [Initializer] instance.
  const Initializer({
    this.debug = false,
    this.group,
    this.outputPath,
    this.order,
  });

  /// parse json into a new [Initializer] instance.
  factory Initializer.fromJson(Map<String, dynamic> json) =>
      _$InitializerFromJson(json);

  /// convert a [Initializer] instance to json.
  Map<String, dynamic> toJson() => _$InitializerToJson(this);
}
